var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import React, { Fragment, cloneElement, isValidElement, useEffect, useMemo, useState } from "react";
import { createPortal } from "react-dom";
import { HiMiniStop, HiVolumeOff, HiVolumeUp } from "./icons.js";
function JSXToText(element) {
    if (isValidElement(element)) {
        const { children } = element.props;
        if (Array.isArray(children))
            return children.map((child) => JSXToText(child));
        return JSXToText(children);
    }
    if (typeof element === "string")
        return element.split(/\s/);
    if (typeof element === "number")
        return [element.toString()];
    return [];
}
function findWordIndex(words, index) {
    let currentIndex = 0;
    function recursiveSearch(subArray, parentIndex) {
        if (subArray.length)
            for (let i = 0; i < subArray.length; i++) {
                const element = subArray[i];
                if (Array.isArray(element)) {
                    const result = recursiveSearch(element, i);
                    if (result !== null)
                        return `${parentIndex === undefined ? "" : parentIndex + "-"}${result}`;
                }
                else {
                    currentIndex += element.length + 1;
                    if (currentIndex > index)
                        return `${parentIndex === undefined ? "" : parentIndex + "-"}${i}`;
                }
            }
        else
            currentIndex++;
        return null;
    }
    return recursiveSearch(words);
}
export default function Speech({ text, id, pitch = 1, rate = 1, volume = 1, lang, voiceURI, startBtn = React.createElement(HiVolumeUp, null), pauseBtn = React.createElement(HiVolumeOff, null), stopBtn = React.createElement(HiMiniStop, null), useStopOverPause, highlightText = false, highlightProps = { style: { fontWeight: "bold" } }, onError = () => alert("Browser not supported! Try some other browser."), props = {}, children, }) {
    const [speechStatus, setSpeechStatus] = useState("stopped");
    const [useStop, setUseStop] = useState();
    const words = useMemo(() => JSXToText(text), [text]);
    const [highlightedIndex, setHighlightedIndex] = useState(null);
    const [highlightContainer, setHighlightContainer] = useState(null);
    const pause = () => { var _a; return speechStatus !== "paused" && ((_a = window.speechSynthesis) === null || _a === void 0 ? void 0 : _a.pause()); };
    const stop = () => { var _a; return speechStatus !== "stopped" && ((_a = window.speechSynthesis) === null || _a === void 0 ? void 0 : _a.cancel()); };
    function start() {
        const synth = window.speechSynthesis;
        if (!synth)
            return onError();
        setSpeechStatus("started");
        if (speechStatus === "paused")
            return synth.resume();
        if (synth.speaking)
            synth.cancel();
        const utterance = new window.SpeechSynthesisUtterance(words.join("\n"));
        utterance.pitch = pitch;
        utterance.rate = rate;
        utterance.volume = volume;
        if (lang)
            utterance.lang = lang;
        if (voiceURI) {
            if (!Array.isArray(voiceURI))
                voiceURI = [voiceURI];
            const voices = synth.getVoices();
            for (let i = 0; i < voiceURI.length; i++) {
                const uri = voiceURI[i];
                const voice = voices.find((voice) => voice.voiceURI === uri);
                if (voice) {
                    utterance.voice = voice;
                    break;
                }
            }
        }
        function setStopped() {
            setSpeechStatus("stopped");
            setHighlightedIndex(null);
            utterance.onpause = null;
            utterance.onend = null;
            utterance.onerror = null;
            utterance.onboundary = null;
            if (synth.paused)
                synth.cancel();
        }
        utterance.onpause = () => setSpeechStatus("paused");
        utterance.onend = setStopped;
        utterance.onerror = setStopped;
        if (highlightText)
            utterance.onboundary = (event) => setHighlightedIndex(findWordIndex(words, event.charIndex));
        synth.speak(utterance);
    }
    function highlightedText(element, parentIndex = "") {
        var _a;
        if (!(highlightedIndex === null || highlightedIndex === void 0 ? void 0 : highlightedIndex.startsWith(parentIndex)))
            return element;
        if (Array.isArray(element))
            return element.map((child, index) => highlightedText(child, parentIndex === "" ? `${index}` : `${parentIndex}-${index}`));
        if (isValidElement(element))
            return cloneElement(element, { key: (_a = element.key) !== null && _a !== void 0 ? _a : Math.random().toString() }, highlightedText(element.props.children, parentIndex));
        if (typeof element === "string" || typeof element === "number") {
            element = element.toString();
            const words = element.split(/\s/);
            const index = +highlightedIndex.split("-").at(-1);
            const before = index ? words.slice(0, index).join(" ").length : -1;
            const highlightedWord = words[index];
            return (React.createElement(Fragment, { key: highlightedIndex },
                element.slice(0, before + 1),
                React.createElement("span", Object.assign({}, highlightProps), highlightedWord),
                element.slice(before + 1 + highlightedWord.length)));
        }
        return element;
    }
    useEffect(() => {
        var _a, _b;
        setUseStop(useStopOverPause !== null && useStopOverPause !== void 0 ? useStopOverPause : (((_a = navigator.userAgentData) === null || _a === void 0 ? void 0 : _a.mobile) || false));
        return (_b = window.speechSynthesis) === null || _b === void 0 ? void 0 : _b.cancel();
    }, []);
    useEffect(() => {
        if (highlightText)
            setHighlightContainer(document.getElementById(`rtts-${id}`));
        else
            setHighlightContainer(null);
    }, [highlightText]);
    return (React.createElement(React.Fragment, null,
        typeof children === "function" ? (children({ speechStatus, start, pause, stop })) : (React.createElement("div", Object.assign({ style: { display: "flex", columnGap: "1rem" } }, props),
            speechStatus !== "started" ? (React.createElement("span", { role: "button", onClick: start }, startBtn)) : useStop === false ? (React.createElement("span", { role: "button", onClick: pause }, pauseBtn)) : (React.createElement("span", { role: "button", onClick: stop }, stopBtn)),
            useStop === false && stopBtn && (React.createElement("span", { role: "button", onClick: stop }, stopBtn)))),
        highlightContainer && createPortal(highlightedText(text), highlightContainer)));
}
export function HighlightedText(_a) {
    var { id, children } = _a, props = __rest(_a, ["id", "children"]);
    const [loading, setLoading] = useState(true);
    useEffect(() => {
        setLoading(false);
    }, []);
    return (React.createElement("div", Object.assign({ id: `rtts-${id}` }, props), loading && children));
}
